using Android.OS;
using Android.Views;
using Android.Widget;
using Com.Syncfusion.Charts;
using Com.Syncfusion.Charts.Enums;
using SilverCoins.BusinessLayer.Managers;
using SilverCoins.BusinessLayer.Models;
using System;
using System.Collections.Generic;
using System.Linq;
using Fragment = Android.Support.V4.App.Fragment;

namespace SilverCoins.Droid.Fragments
{
    public class StatisticsFragment : Fragment
    {
        private View view;
        private SfChart chart;
        private Spinner spinnerAccounts;
        private List<Account> listOfAccounts = SilverCoinsManager.GetAccounts().ToList();
        private Account account;

        public override View OnCreateView(LayoutInflater inflater, ViewGroup container, Bundle savedInstanceState)
        {
            HasOptionsMenu = true;

            listOfAccounts.Insert(0, new Account() { Name = "All accounts", Id = 0 });
            view = inflater.Inflate(Resource.Layout.fragment_statistics, null);

            spinnerAccounts = Activity.FindViewById<Spinner>(Resource.Id.spinnerAccounts);
            spinnerAccounts.ItemSelected += new EventHandler<AdapterView.ItemSelectedEventArgs>(SpinnerTransactionAccount_ItemClick);

            account = listOfAccounts[spinnerAccounts.SelectedItemPosition];

            chart = view.FindViewById<SfChart>(Resource.Id.barChartCategoryTypes);

            ShowLineChart();

            return view;
        }

        private void SpinnerTransactionAccount_ItemClick(object sender, AdapterView.ItemSelectedEventArgs e)
        {
            account = listOfAccounts[spinnerAccounts.SelectedItemPosition];
            switch (chart.Title.Text)
            {
                case "Income & Expenses":
                    ShowPieChart();
                    break;

                case "Cash flow by type":
                    ShowColumnChart();
                    break;

                case "Balance":
                    ShowLineChart();
                    break;

                default:
                    break;
            }
        }

        public override void OnCreateOptionsMenu(IMenu menu, MenuInflater inflater)
        {
            base.OnCreateOptionsMenu(menu, inflater);
            inflater.Inflate(Resource.Menu.menu_statistics, menu);
        }

        public override bool OnOptionsItemSelected(IMenuItem item)
        {
            switch (item.ItemId)
            {
                case Resource.Id.action_piechart:
                    ShowPieChart();
                    return true;

                case Resource.Id.action_linechart:
                    ShowColumnChart();
                    return true;

                case Resource.Id.action_barchart:
                    ShowLineChart();
                    return true;

                default:
                    return base.OnOptionsItemSelected(item);
            }
        }

        private void ShowPieChart()
        {
            chart.Series.Clear();
            chart.Title.Text = "Income & Expenses";
            chart.Legend.Visibility = Visibility.Visible;

            var dataModel = Statistics.Statistics.PieChartData(account);

            PieSeries pieSeries = new PieSeries()
            {
                DataSource = dataModel,
                DataPointSelectionEnabled = true,
                DataMarkerPosition = CircularSeriesDataMarkerPosition.Outside,
                AnimationEnabled = true,
                ExplodableOnTouch = true,
                SmartLabelsEnabled = true,
                Visibility = Visibility.Visible,
                TooltipEnabled = true,
            };

            chart.Series.Add(pieSeries);
        }

        private void ShowColumnChart()
        {
            chart.Series.Clear();
            chart.Title.Text = "Cash flow by type";
            chart.Legend.Visibility = Visibility.Visible;

            CategoryAxis primaryAxis = new CategoryAxis();
            primaryAxis.Title.Text = "Month";
            chart.PrimaryAxis = primaryAxis;

            //Initializing Secondary Axis
            NumericalAxis secondaryAxis = new NumericalAxis();
            secondaryAxis.Title.Text = "Amount";
            chart.SecondaryAxis = secondaryAxis;

            var dataModel = Statistics.Statistics.ColumnChartData(account);

            ColumnSeries areaSeries1 = new ColumnSeries()
            {
                DataSource = dataModel[0],
                TooltipEnabled = true,
                AnimationEnabled = true,
                AnimationDuration = 0.2,
                Label = "Income",
            };

            chart.Series.Add(areaSeries1);

            ColumnSeries areaSeries2 = new ColumnSeries()
            {
                DataSource = dataModel[1],
                TooltipEnabled = true,
                AnimationEnabled = true,
                AnimationDuration = 0.2,
                Label = "Expenses",
            };

            chart.Series.Add(areaSeries2);
        }

        private void ShowLineChart()
        {
            chart.Series.Clear();
            chart.Title.Text = "Balance";
            chart.Legend.Visibility = Visibility.Visible;

            CategoryAxis primaryAxis = new CategoryAxis();
            primaryAxis.Title.Text = "Month";
            chart.PrimaryAxis = primaryAxis;

            NumericalAxis secondaryAxis = new NumericalAxis();
            secondaryAxis.Title.Text = "Amount (in lakh)";
            chart.SecondaryAxis = secondaryAxis;

            ObservableArrayList data = new ObservableArrayList();
            data.Add(new ChartDataPoint("Jan", 28));
            data.Add(new ChartDataPoint("Feb", 35));
            data.Add(new ChartDataPoint("Mar", 22));
            data.Add(new ChartDataPoint("Apr", 30));

            ObservableArrayList data1 = new ObservableArrayList();
            data1.Add(new ChartDataPoint("Jan", 38));
            data1.Add(new ChartDataPoint("Feb", 25));
            data1.Add(new ChartDataPoint("Mar", 12));
            data1.Add(new ChartDataPoint("Apr", 33));

            ColumnSeries Series1 = new ColumnSeries()
            {
                DataSource = data,
                TooltipEnabled = true,
                AnimationEnabled = true,
                AnimationDuration = 0.2,
                Label = "Income",
            };

            ColumnSeries Series2 = new ColumnSeries()
            {
                DataSource = data1,
                TooltipEnabled = true,
                AnimationEnabled = true,
                AnimationDuration = 0.2,
                Label = "Expenses",
            };
            chart.Series.Add(Series1);
            chart.Series.Add(Series2);
        }
    }
}